@@ -119,6 +119,36 @@ void uwsgi_log_verbose(const char *fmt, ...) {
 }
 
 
+/*
+	commodity function mainly useful in log rotation
+*/
+void uwsgi_logfile_write(const char *fmt, ...) {
+	va_list ap;
+        char logpkt[4096];
+
+        struct timeval tv;
+        char ctime_storage[26];
+
+	gettimeofday(&tv, NULL);
+#if defined(__sun__) && !defined(__clang__)
+        ctime_r((const time_t *) &tv.tv_sec, ctime_storage, 26);
+#else
+        ctime_r((const time_t *) &tv.tv_sec, ctime_storage);
+#endif
+        memcpy(logpkt, ctime_storage, 24);
+        memcpy(logpkt + 24, " - ", 3);
+
+        int rlen = 24 + 3;
+
+        va_start(ap, fmt);
+        rlen += vsnprintf(logpkt + rlen, 4096 - rlen, fmt, ap);
+        va_end(ap);
+
+        // do not check for errors
+        rlen = write(uwsgi.original_log_fd, logpkt, rlen);
+}
+
+
 
 static void fix_logpipe_buf(int *fd) {
 	int so_bufsize;
@@ -476,7 +506,6 @@ void uwsgi_check_logrotate(void) {
 }
 
 void uwsgi_log_rotate() {
-	char message[1024];
 	if (!uwsgi.logfile) return;
 	char *rot_name = uwsgi.log_backupname;
                 int need_free = 0;
@@ -486,24 +515,21 @@ void uwsgi_log_rotate() {
                         free(ts_str);
                         need_free = 1;
                 }
-                int ret = snprintf(message, 1024, "[%d] logsize: %llu, triggering rotation to %s...\n", (int) uwsgi_now(), (unsigned long long) uwsgi.shared->logsize, rot_name);
-                if (ret > 0) {
-                        if (write(uwsgi.original_log_fd, message, ret) != ret) {
-                                // very probably this will never be printed
-                                uwsgi_error("write()");
-                        }
-                }
+		// this will be rawly written to the logfile
+                uwsgi_logfile_write("logsize: %llu, triggering rotation to %s...\n", (unsigned long long) uwsgi.shared->logsize, rot_name);
                 if (rename(uwsgi.logfile, rot_name) == 0) {
-                        // reopen logfile dup'it and eventually gracefully reload workers;
+                        // reopen logfile and dup'it, on dup2 error, exit(1)
                         int fd = open(uwsgi.logfile, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP);
                         if (fd < 0) {
+				// this will be written to the original file
                                 uwsgi_error_open(uwsgi.logfile);
-                                grace_them_all(0);
+				exit(1);
                         }
                         else {
                                 if (dup2(fd, uwsgi.original_log_fd) < 0) {
-                                        uwsgi_error("dup2()");
-                                        grace_them_all(0);
+					// this could be lost :(
+                                        uwsgi_error("uwsgi_log_rotate()/dup2()");
+					exit(1);
                                 }
                                 close(fd);
                         }
@@ -519,7 +545,7 @@ void uwsgi_log_reopen() {
 	char message[1024];
 	if (!uwsgi.logfile) return;
 	int ret = snprintf(message, 1024, "[%d] logsize: %llu, triggering log-reopen...\n", (int) uwsgi_now(), (unsigned long long) uwsgi.shared->logsize);
-        if (ret > 0) {
+        if (ret > 0 && ret < 1024) {
                         if (write(uwsgi.original_log_fd, message, ret) != ret) {
                                 // very probably this will never be printed
                                 uwsgi_error("write()");
@@ -535,7 +561,7 @@ void uwsgi_log_reopen() {
 			return;
                 }
                 ret = snprintf(message, 1024, "[%d] %s reopened.\n", (int) uwsgi_now(), uwsgi.logfile);
-                if (ret > 0) {
+                if (ret > 0 && ret < 1024) {
                         if (write(uwsgi.original_log_fd, message, ret) != ret) {
                                 // very probably this will never be printed
                                 uwsgi_error("write()");
@@ -547,7 +573,7 @@ void uwsgi_log_reopen() {
 
 void log_request(struct wsgi_request *wsgi_req) {
 
-	int log_it = uwsgi.shared->options[UWSGI_OPTION_LOGGING];
+	int log_it = uwsgi.logging_options.enabled;
 
 	if (wsgi_req->do_not_log)
 		return;
@@ -557,25 +583,25 @@ void log_request(struct wsgi_request *wsgi_req) {
 	}
 
 /* conditional logging */
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_ZERO] && wsgi_req->response_size == 0) {
+	if (uwsgi.logging_options.zero && wsgi_req->response_size == 0) {
 		goto logit;
 	}
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_SLOW] && (uint32_t) wsgi_req_time >= uwsgi.shared->options[UWSGI_OPTION_LOG_SLOW]) {
+	if (uwsgi.logging_options.slow && (uint32_t) wsgi_req_time >= uwsgi.logging_options.slow) {
 		goto logit;
 	}
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_4xx] && (wsgi_req->status >= 400 && wsgi_req->status <= 499)) {
+	if (uwsgi.logging_options._4xx && (wsgi_req->status >= 400 && wsgi_req->status <= 499)) {
 		goto logit;
 	}
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_5xx] && (wsgi_req->status >= 500 && wsgi_req->status <= 599)) {
+	if (uwsgi.logging_options._5xx && (wsgi_req->status >= 500 && wsgi_req->status <= 599)) {
 		goto logit;
 	}
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_BIG] && (wsgi_req->response_size >= uwsgi.shared->options[UWSGI_OPTION_LOG_BIG])) {
+	if (uwsgi.logging_options.big && (wsgi_req->response_size >= uwsgi.logging_options.big)) {
 		goto logit;
 	}
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_SENDFILE] && wsgi_req->via == UWSGI_VIA_SENDFILE) {
+	if (uwsgi.logging_options.sendfile && wsgi_req->via == UWSGI_VIA_SENDFILE) {
 		goto logit;
 	}
-	if (uwsgi.shared->options[UWSGI_OPTION_LOG_IOERROR] && wsgi_req->read_errors > 0 && wsgi_req->write_errors > 0) {
+	if (uwsgi.logging_options.ioerror && wsgi_req->read_errors > 0 && wsgi_req->write_errors > 0) {
 		goto logit;
 	}
 
@@ -660,7 +686,7 @@ void uwsgi_logit_simple(struct wsgi_request *wsgi_req) {
 		logvecpos++;
 	}
 
-	if (uwsgi.shared->options[UWSGI_OPTION_MEMORY_DEBUG] == 1) {
+	if (uwsgi.logging_options.memory_report == 1) {
 		rlen = snprintf(mempkt, 4096, "{address space usage: %lld bytes/%lluMB} {rss usage: %llu bytes/%lluMB} ", (unsigned long long) uwsgi.workers[uwsgi.mywid].vsz_size, (unsigned long long) uwsgi.workers[uwsgi.mywid].vsz_size / 1024 / 1024,
 			(unsigned long long) uwsgi.workers[uwsgi.mywid].rss_size, (unsigned long long) uwsgi.workers[uwsgi.mywid].rss_size / 1024 / 1024);
 		logvec[logvecpos].iov_base = mempkt;
