@@ -407,6 +407,7 @@ void uwsgi_reload(char **argv) {
 	/* check fd table (a module can obviosly open some fd on initialization...) */
 	uwsgi_log("closing all non-uwsgi socket fds > 2 (max_fd = %d)...\n", (int) uwsgi.max_fd);
 	for (i = 3; i < (int) uwsgi.max_fd; i++) {
+		if (uwsgi.close_on_exec2) fcntl(i, F_SETFD, 0);
 
 		if (uwsgi_fd_is_safe(i)) continue;
 
@@ -587,6 +588,8 @@ void uwsgi_fixup_fds(int wid, int muleid, struct uwsgi_gateway *ug) {
 
 		if (uwsgi.master_fifo_fd > -1) close(uwsgi.master_fifo_fd);
 
+		if (uwsgi.notify_socket_fd > -1) close(uwsgi.notify_socket_fd);
+
 #ifdef __linux__
 		for(i=0;i<uwsgi.setns_fds_count;i++) {
 			close(uwsgi.setns_fds[i]);
