@@ -754,7 +754,11 @@ void uwsgi_as_root() {
 	}
 
 	if (uwsgi.logfile_chown) {
-		if (fchown(2, uwsgi.uid, uwsgi.gid)) {
+		int log_fd = 2;
+		if (uwsgi.log_master && uwsgi.original_log_fd > -1) {
+			log_fd = uwsgi.original_log_fd;
+		}
+		if (fchown(log_fd, uwsgi.uid, uwsgi.gid)) {
 			uwsgi_error("fchown()");
 			exit(1);
 		}
@@ -1432,7 +1436,7 @@ void uwsgi_heartbeat() {
 		return;
 
 	time_t now = uwsgi_now();
-	if (uwsgi.next_heartbeat < now) {
+	if (uwsgi.next_heartbeat <= now) {
 		char byte = 26;
 		if (write(uwsgi.emperor_fd, &byte, 1) != 1) {
 			uwsgi_error("write()");
@@ -1456,7 +1460,15 @@ int wsgi_req_accept(int queue, struct wsgi_request *wsgi_req) {
 	// heartbeat
 	// in multithreaded mode we are now locked
 	if (uwsgi.has_emperor && uwsgi.heartbeat) {
+		time_t now = uwsgi_now();
+		// overengineering ... (reduce skew problems)
 		timeout = uwsgi.heartbeat;
+		if (!uwsgi.next_heartbeat) {
+			uwsgi.next_heartbeat = now;
+		}
+		if (uwsgi.next_heartbeat >= now) {
+			timeout = uwsgi.next_heartbeat - now;
+		}
 	}
 
 	// need edge trigger ?
@@ -1479,7 +1491,7 @@ int wsgi_req_accept(int queue, struct wsgi_request *wsgi_req) {
 	}
 
 	// check for heartbeat
-	if (timeout > 0) {
+	if (uwsgi.has_emperor && uwsgi.heartbeat) {
 		uwsgi_heartbeat();
 		// no need to continue if timed-out
 		if (ret == 0) {
@@ -4182,7 +4194,8 @@ char *uwsgi_str_to_hex(char *src, size_t slen) {
 	char *ptr = dst;
 	size_t i;
 	for (i = 0; i < slen; i++) {
-		memcpy(ptr, uwsgi_hex_table[(int) src[i]], 2);
+		uint8_t pos = (uint8_t) src[i];
+		memcpy(ptr, uwsgi_hex_table[pos], 2);
 		ptr += 2;
 	}
 	return dst;
