@@ -107,11 +107,67 @@ XS(XS_input_read) {
         SV *read_buf = ST(1);
         unsigned long arg_len = SvIV(ST(2));
 
+	long offset = 0;
+	if (items > 2) {
+		offset = (long) SvIV(ST(3));
+	}
+
 	ssize_t rlen = 0;
 
 	char *buf = uwsgi_request_body_read(wsgi_req, arg_len, &rlen);
         if (buf) {
-		sv_setpvn(read_buf, buf, rlen);
+		if (rlen > 0 && offset != 0) {
+			STRLEN orig_len;
+			// get data from original string
+        		char *orig = SvPV(read_buf, orig_len);	
+			size_t new_size = orig_len;
+			// still valid ?
+			if (offset > 0) {
+				// if the new string is bigger than the old one, allocate a bigger chunk
+				if ((size_t) rlen + offset > orig_len) {
+					new_size = rlen + offset;
+				}
+				// if offset is bigger than orig_len, pad with "\0", so we use (slower) calloc
+				char *new_buf = uwsgi_calloc(new_size);
+				// put back older value
+				memcpy(new_buf, orig, orig_len);
+				// put the new value
+				memcpy(new_buf + offset, buf, rlen);
+				sv_setpvn(read_buf, new_buf, new_size);
+				// free the new value
+				free(new_buf);
+			}
+			// negative (a little bit more complex)
+			else {
+				long orig_offset = 0;
+				 // first of all get the new orig_len;   
+                                offset = abs(offset);
+                                if (offset > (long) orig_len) {
+                                        new_size = offset;
+					orig_offset = offset - orig_len;
+                                        offset = 0;
+                                }
+                                else {
+                                        offset = orig_len - offset;
+                                }
+
+				if ((size_t) rlen + offset > new_size) {
+					new_size = rlen + offset;
+				}
+
+				char *new_buf = uwsgi_calloc(new_size);
+				// put back older value
+                                memcpy(new_buf + orig_offset, orig, orig_len);
+				 // put the new value
+                                memcpy(new_buf + offset, buf, rlen);
+				sv_setpvn(read_buf, new_buf, new_size);
+				// free the new value
+				free(new_buf);
+			}
+		}
+		else {
+			sv_setpvn(read_buf, buf, rlen);
+		}
 		goto ret;
         }
 
@@ -171,20 +227,6 @@ XS(XS_error_print) {
         XSRETURN(0);
 }
 
-XS(XS_uwsgi_stacktrace) {
-
-	dXSARGS;
-
-        psgi_check_args(0);
-	uwsgi_log("%s", SvPV_nolen(ERRSV));
-	uwsgi_log("*** uWSGI perl stacktrace ***\n");
-	SV *ret = perl_eval_pv("Devel::StackTrace->new->as_string;", 0);
-        uwsgi_log("%s", SvPV_nolen(ret));
-	uwsgi_log("*** end of perl stacktrace ***\n\n");
-	XSRETURN(0);
-
-}
-
 
 /* automatically generated */
 
@@ -197,6 +239,7 @@ xs_init(pTHX)
 {
         char *file = __FILE__;
         dXSUB_SYS;
+	HV *stash;
 
         /* DynaLoader is a special case */
         newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
@@ -218,13 +261,16 @@ xs_init(pTHX)
         newXS("uwsgi::streaming::write", XS_streaming_write, "uwsgi::streaming");
         newXS("uwsgi::streaming::close", XS_streaming_close, "uwsgi::streaming");
 
-        newXS("uwsgi::stacktrace", XS_uwsgi_stacktrace, "uwsgi");
-
-
         uperl.tmp_streaming_stash[uperl.tmp_current_i] = gv_stashpv("uwsgi::streaming", 0);
 
 nonworker:
 
+	stash = gv_stashpv("uwsgi", 1);
+	newCONSTSUB(stash, "VERSION", newSVpv(UWSGI_VERSION, 0));
+	newCONSTSUB(stash, "SPOOL_OK", newSViv(-2));
+	newCONSTSUB(stash, "SPOOL_RETRY", newSViv(-1));
+	newCONSTSUB(stash, "SPOOL_IGNORE", newSViv(0));
+
         init_perl_embedded_module();
 
 }
@@ -347,16 +393,12 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
         		}
 		}
 
-		perl_eval_pv("use IO::Handle;", 0);
-		perl_eval_pv("use IO::File;", 0);
-		perl_eval_pv("use IO::Socket;", 0);
-		perl_eval_pv("use Scalar::Util;", 0);
+		perl_eval_pv("use IO::Handle;", 1);
+		perl_eval_pv("use IO::File;", 1);
+		perl_eval_pv("use IO::Socket;", 1);
+		perl_eval_pv("use Scalar::Util;", 1);
 		if (!uperl.no_die_catch) {
-			perl_eval_pv("use Devel::StackTrace;", 0);
-			if (!SvTRUE(ERRSV)) {
-				uperl.stacktrace_available = 1;
-				perl_eval_pv("$SIG{__DIE__} = \\&uwsgi::stacktrace;", 0);
-			}
+			perl_eval_pv("use Devel::StackTrace; $SIG{__DIE__} = sub { print Devel::StackTrace->new()->as_string() };", 0);
 		}
 
 		if (uperl.argv_items || uperl.argv_item) {
@@ -394,7 +436,7 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
 	free(buf);
 
 	if(SvTRUE(ERRSV)) {
-        	uwsgi_log("%s\n", SvPV_nolen(ERRSV));
+        	uwsgi_log("%s", SvPV_nolen(ERRSV));
 		free(callables);
 		uwsgi_perl_free_stashes();
 		goto clear;
@@ -456,6 +498,17 @@ clear2:
        	return -1; 
 }
 
+void uwsgi_psgi_preinit_apps() {
+	if (uperl.exec) {
+		PERL_SET_CONTEXT(uperl.main[0]);
+                perl_parse(uperl.main[0], xs_init, 3, uperl.embedding, NULL);
+		struct uwsgi_string_list *usl;
+        	uwsgi_foreach(usl, uperl.exec) {
+                	uwsgi_perl_exec(usl->value);
+        	}
+	}
+}
+
 void uwsgi_psgi_app() {
 
         if (uperl.psgi) {
@@ -463,15 +516,11 @@ void uwsgi_psgi_app() {
 		init_psgi_app(NULL, uperl.psgi, strlen(uperl.psgi), uperl.main);
         }
 	// create a perl environment (if needed)
-	else if (uperl.exec || uperl.shell) {
+	else if (!uperl.exec && uperl.shell) {
 		PERL_SET_CONTEXT(uperl.main[0]);
-                perl_parse(uperl.main[0], xs_init, 2, uperl.embedding, NULL);
+                perl_parse(uperl.main[0], xs_init, 3, uperl.embedding, NULL);
 	}
 
-	struct uwsgi_string_list *usl;
-        uwsgi_foreach(usl, uperl.exec) {
-                uwsgi_perl_exec(usl->value);
-        }
 
 }
 
@@ -480,7 +529,7 @@ int uwsgi_perl_mule(char *opt) {
         if (uwsgi_endswith(opt, ".pl")) {
                 PERL_SET_CONTEXT(uperl.main[0]);
                 uperl.embedding[1] = opt;
-                if (perl_parse(uperl.main[0], xs_init, 2, uperl.embedding, NULL)) {
+                if (perl_parse(uperl.main[0], xs_init, 3, uperl.embedding, NULL)) {
                         return 0;
                 }
                 perl_run(uperl.main[0]);
@@ -495,6 +544,6 @@ int uwsgi_perl_mule(char *opt) {
 void uwsgi_perl_exec(char *filename) {
 	size_t size = 0;
         char *buf = uwsgi_open_and_read(filename, &size, 1, NULL);
-        perl_eval_pv(buf, 0);
+        perl_eval_pv(buf, 1);
 	free(buf);
 }
