@@ -16,12 +16,15 @@ struct uwsgi_pypy {
 	char *home;
 	char *wsgi;
 	char *wsgi_file;
+	char *paste;
 	struct uwsgi_string_list *eval;
 	struct uwsgi_string_list *eval_post_fork;
 	struct uwsgi_string_list *exec;
 	struct uwsgi_string_list *exec_post_fork;
 
 	struct uwsgi_string_list *pp;
+
+	pthread_mutex_t attach_thread_lock;
 } upypy;
 
 // the functions exposed by libpypy-c
@@ -34,6 +37,7 @@ void (*u_pypy_init_threads)(void);
 // the hooks you can override with pypy
 void (*uwsgi_pypy_hook_loader)(char *);
 void (*uwsgi_pypy_hook_file_loader)(char *);
+void (*uwsgi_pypy_hook_paste_loader)(char *);
 void (*uwsgi_pypy_hook_pythonpath)(char *);
 void (*uwsgi_pypy_hook_request)(void *, int);
 void (*uwsgi_pypy_post_fork_hook)(void);
@@ -231,6 +235,10 @@ static void uwsgi_pypy_init_apps() {
 	if (uwsgi_pypy_hook_file_loader && upypy.wsgi_file) {
 		uwsgi_pypy_hook_file_loader(upypy.wsgi_file);
 	}
+
+	if (uwsgi_pypy_hook_paste_loader && upypy.paste) {
+		uwsgi_pypy_hook_paste_loader(upypy.paste);
+	}
 }
 
 /*
@@ -240,12 +248,20 @@ static void uwsgi_pypy_atexit() {
 }
 */
 
+static void uwsgi_opt_pypy_ini_paste(char *opt, char *value, void *foobar) {
+        uwsgi_opt_load_ini(opt, value, NULL);
+        upypy.paste = value;
+}
+
+
 static struct uwsgi_option uwsgi_pypy_options[] = {
 	{"pypy-lib", required_argument, 0, "set the path/name of the pypy library", uwsgi_opt_set_str, &upypy.lib, 0},
 	{"pypy-setup", required_argument, 0, "set the path of the python setup script", uwsgi_opt_set_str, &upypy.setup, 0},
 	{"pypy-home", required_argument, 0, "set the home of pypy library", uwsgi_opt_set_str, &upypy.home, 0},
 	{"pypy-wsgi", required_argument, 0, "load a WSGI module", uwsgi_opt_set_str, &upypy.wsgi, 0},
 	{"pypy-wsgi-file", required_argument, 0, "load a WSGI/mod_wsgi file", uwsgi_opt_set_str, &upypy.wsgi_file, 0},
+	{"pypy-ini-paste", required_argument, 0, "load a paste.deploy config file containing uwsgi section", uwsgi_opt_pypy_ini_paste, NULL, UWSGI_OPT_IMMEDIATE},
+	{"pypy-paste", required_argument, 0, "load a paste.deploy config file", uwsgi_opt_set_str, &upypy.paste, 0},
 	{"pypy-eval", required_argument, 0, "evaluate pypy code before fork()", uwsgi_opt_add_string_list, &upypy.eval, 0},
 	{"pypy-eval-post-fork", required_argument, 0, "evaluate pypy code soon after fork()", uwsgi_opt_add_string_list, &upypy.eval_post_fork, 0},
 	{"pypy-exec", required_argument, 0, "execute pypy code from file before fork()", uwsgi_opt_add_string_list, &upypy.exec, 0},
@@ -264,7 +280,9 @@ static void uwsgi_pypy_enable_threads() {
 
 static void uwsgi_pypy_init_thread() {
 	if (u_pypy_thread_attach) {
+		pthread_mutex_lock(&upypy.attach_thread_lock);
 		u_pypy_thread_attach();
+		pthread_mutex_unlock(&upypy.attach_thread_lock);
 	}
 }
 
@@ -286,6 +304,7 @@ static uint64_t uwsgi_pypy_rpc(void *func, uint8_t argc, char **argv, uint16_t a
 }
 
 static void uwsgi_pypy_post_fork() {
+	pthread_mutex_init(&upypy.attach_thread_lock, NULL);
 	struct uwsgi_string_list *usl = upypy.eval_post_fork;
         while(usl) {
                 if (u_pypy_execute_source(usl->value)) {
