@@ -26,6 +26,7 @@ int close(int);
 
 void (*uwsgi_pypy_hook_loader)(char *);
 void (*uwsgi_pypy_hook_file_loader)(char *);
+void (*uwsgi_pypy_hook_paste_loader)(char *);
 void (*uwsgi_pypy_hook_pythonpath)(char *);
 void (*uwsgi_pypy_hook_request)(struct wsgi_request *);
 void (*uwsgi_pypy_post_fork_hook)(void);
@@ -304,6 +305,25 @@ def uwsgi_pypy_file_loader(filename):
     mod = imp.load_source('uwsgi_file_wsgi', w)
     wsgi_application = getattr(mod, c)
 
+"""
+load a .ini paste app
+"""
+@ffi.callback("void(char *)")
+def uwsgi_pypy_paste_loader(config):
+    global wsgi_application
+    c = ffi.string(config)
+    if c.startswith('config:'):
+        c = c[7:]
+    if c[0] != '/':
+        c = os.getcwd() + '/' + c
+    try:
+        from paste.script.util.logging_config import fileConfig
+        fileConfig(c)
+    except ImportError:
+        print "PyPy WARNING: unable to load paste.script.util.logging_config"
+    from paste.deploy import loadapp
+    wsgi_application = loadapp('config:%s' % c)
+
 """
 .post_fork_hook
 """
@@ -462,6 +482,7 @@ def uwsgi_pypy_wsgi_handler(wsgi_req):
 
 lib.uwsgi_pypy_hook_loader = uwsgi_pypy_loader
 lib.uwsgi_pypy_hook_file_loader = uwsgi_pypy_file_loader
+lib.uwsgi_pypy_hook_paste_loader = uwsgi_pypy_paste_loader
 lib.uwsgi_pypy_hook_pythonpath = uwsgi_pypy_pythonpath
 lib.uwsgi_pypy_hook_request = uwsgi_pypy_wsgi_handler
 lib.uwsgi_pypy_post_fork_hook = uwsgi_pypy_post_fork_hook
