@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '2.0'
+uwsgi_version = '2.0.1'
 
 import os
 import re
@@ -244,7 +244,10 @@ def build_uwsgi(uc, print_only=False, gcll=None):
         ep = uc.get('embedded_plugins').split(',')
         epc = "-DUWSGI_DECLARE_EMBEDDED_PLUGINS=\""
         eplc = "-DUWSGI_LOAD_EMBEDDED_PLUGINS=\""
-        for p in ep:
+        for item in ep:
+            # allow name=path syntax
+            kv = item.split('=')
+            p = kv[0]
             if p is None or p == 'None':
                 continue
             p = p.strip()
@@ -358,15 +361,25 @@ def build_uwsgi(uc, print_only=False, gcll=None):
 
         if len(ep) > 0:
             push_print("*** uWSGI compiling embedded plugins ***")
-            for p in ep:
+            for item in ep:
+                # allows name=path syntax
+                kv = item.split('=')
+                if len(kv) > 1:
+                    p = kv[1]
+                    p = p.strip()
+                    path = os.path.abspath(p)
+                else:
+                    p = kv[0]
+                    p = p.strip()
+                    path = 'plugins/%s' % p
+
                 if p is None or p == 'None':
                     continue
-                p = p.strip()
 
                 if p == 'ugreen':
                     if uwsgi_os == 'OpenBSD' or uwsgi_cpu[0:3] == 'arm' or uwsgi_os == 'Haiku' or uwsgi_os.startswith('CYGWIN') or (uwsgi_os == 'Darwin' and uwsgi_os_k.startswith('8')):
                         continue
-                path = 'plugins/%s' % p
+
                 path = path.rstrip('/')
 
                 if not os.path.isdir(path):
@@ -493,9 +506,9 @@ def build_uwsgi(uc, print_only=False, gcll=None):
     bin_name = os.environ.get('UWSGI_BIN_NAME', uc.get('bin_name'))
 
     if uc.embed_config:
-        gcc_list.append(uc.embed_config)
+        gcc_list.append("%s.o" % binarize(uc.embed_config))
     for ef in binary_list:
-        gcc_list.append("build/%s" % ef)
+        gcc_list.append("%s.o" % ef)
 
     additional_sources = os.environ.get('UWSGI_ADDITIONAL_SOURCES')
     if not additional_sources:
@@ -734,7 +747,7 @@ class uConf(object):
         if 'UWSGI_PROFILE_OVERRIDE' in os.environ:
             for item in os.environ['UWSGI_PROFILE_OVERRIDE'].split(';'):
                 k,v = item.split('=', 2)
-                uc.set(k, v)
+                self.set(k, v)
 
         if 'UWSGI_AS_LIB' in os.environ:
             self.set('as_shared_library', 'true')
@@ -1041,13 +1054,16 @@ class uConf(object):
             if not self.embed_config:
                 self.embed_config = self.get('embed_config')
             if self.embed_config:
-                binary_link_cmd = "ld -r -b binary -o %s.o %s" % (self.embed_config, self.embed_config)
+                binary_link_cmd = "ld -r -b binary -o %s.o %s" % (binarize(self.embed_config), self.embed_config)
                 print(binary_link_cmd)
                 os.system(binary_link_cmd)
-                self.cflags.append("-DUWSGI_EMBED_CONFIG=_binary_%s_start" % self.embed_config.replace('.','_'))
-                self.cflags.append("-DUWSGI_EMBED_CONFIG_END=_binary_%s_end" % self.embed_config.replace('.','_'))
-            if self.get('embed_files'):
-                for ef in self.get('embed_files').split(','):
+                self.cflags.append("-DUWSGI_EMBED_CONFIG=_binary_%s_start" % binarize(self.embed_config))
+                self.cflags.append("-DUWSGI_EMBED_CONFIG_END=_binary_%s_end" % binarize(self.embed_config))
+            embed_files = os.environ.get('UWSGI_EMBED_FILES')
+            if not embed_files:
+                embed_files = self.get('embed_files')
+            if embed_files:
+                for ef in embed_files.split(','):
                     ef_parts = ef.split('=')
                     symbase = None
                     if len(ef_parts) > 1:
@@ -1057,7 +1073,7 @@ class uConf(object):
                         for directory, directories, files in os.walk(ef):
                             for f in files:
                                 fname = "%s/%s" % (directory, f)
-                                binary_link_cmd = "ld -r -b binary -o build/%s.o %s" % (binarize(fname), fname)
+                                binary_link_cmd = "ld -r -b binary -o %s.o %s" % (binarize(fname), fname)
                                 print(binary_link_cmd)
                                 os.system(binary_link_cmd)
                                 if symbase:
@@ -1067,7 +1083,7 @@ class uConf(object):
                                         os.system(objcopy_cmd)
                                 binary_list.append(binarize(fname))
                     else:
-                        binary_link_cmd = "ld -r -b binary -o build/%s.o %s" % (binarize(ef), ef)
+                        binary_link_cmd = "ld -r -b binary -o %s.o %s" % (binarize(ef), ef)
                         print(binary_link_cmd)
                         os.system(binary_link_cmd)
                         binary_list.append(binarize(ef))
@@ -1117,7 +1133,7 @@ class uConf(object):
                 report['yaml'] = 'libyaml'
 
         if self.get('json'):
-            if self.get('json') == 'auto':
+            if self.get('json') in ('auto', 'true'):
                 jsonconf = spcall("pkg-config --cflags jansson")
                 if jsonconf:
                     self.cflags.append(jsonconf)
@@ -1139,6 +1155,9 @@ class uConf(object):
                         self.libs.append(spcall("pkg-config --libs yajl"))
                         self.cflags.append("-DUWSGI_JSON_YAJL")
                         report['json'] = 'yajl'
+                    elif self.get('json') == 'true':
+                        print("*** jansson and yajl headers unavailable. uWSGI build is interrupted. You have to install jansson or yajl development headers or disable JSON")
+                        sys.exit(1)
             elif self.get('json') == 'jansson':
                 jsonconf = spcall("pkg-config --cflags jansson")
                 if jsonconf:
@@ -1164,6 +1183,18 @@ class uConf(object):
                     self.libs.append(spcall("pkg-config --libs yajl"))
                     self.cflags.append("-DUWSGI_JSON_YAJL")
                     report['json'] = 'yajl'
+                elif self.has_include('yajl/yajl_tree.h'):
+                    self.cflags.append("-DUWSGI_JSON")
+                    self.gcc_list.append('core/json')
+                    self.libs.append('-lyajl')
+                    self.cflags.append("-DUWSGI_JSON_YAJL")
+                    report['json'] = 'yajl'
+                elif self.has_include('yajl/yajl_parse.h'):
+                    self.cflags.append("-DUWSGI_JSON")
+                    self.gcc_list.append('core/json')
+                    self.libs.append('-lyajl')
+                    self.cflags.append("-DUWSGI_JSON_YAJL_OLD")
+                    report['json'] = 'yajl_old'
                 else:
                     print("*** yajl headers unavailable. uWSGI build is interrupted. You have to install yajl development package or use jansson or disable JSON")
                     sys.exit(1)
